import fs from 'fs';
import tmp from 'tmp';
import path, { join, dirname } from 'path';

// src/replacements.coffee
var replacements = [
  {
    from: 'Run ncu with -u to upgrade package.json',
    to: "Run 'sake outdated:update' to update your package.json"
  },
  {
    from: 'The following dependencies are satisfied by their declared version range, but the installed versions are behind. You can install the latest versions without modifying your package file by using npm update. If you want to update the dependencies in your package file anyway, run ncu -a.',
    to: "The following dependencies are satisfied by their declared version ranges. Run\n'sake outdated:all' to update your package.json to the latest versions."
  },
  {
    from: 'The following dependency is satisfied by its declared version range, but the installed version is behind. You can install the latest version without modifying your package file by using npm update. If you want to update the dependency in your package file anyway, run ncu -a.',
    to: "The following dependency is satisfied by its declared version range. Run 'sake\noutdated:all' to update your package.json to the latest version instead."
  },
  {
    from: 'All dependencies match the latest package versions :)',
    to: 'All dependencies up to date'
  },
  {
    from: 'Upgraded /',
    to: '\nUpdated /'
  }
];

// src/utils.coffee

// Log helper to replace messages with our defaults
var log = function(stdout, stderr) {
  var from, j, len, to;
  stdout = stdout.trim();
  stderr = stderr.trim();
  for (j = 0, len = replacements.length; j < len; j++) {
    ({from, to} = replacements[j]);
    stdout = stdout.replace(from, to);
  }
  if (stdout) {
    console.log(stdout);
  }
  if (stderr) {
    return console.error(stderr);
  }
};

// Split stdout lines, skipping header/footer text
var splitLines = function(stdout) {
  var i, j, k, len, len1, line, lines;
  lines = stdout.trim().split('\n');
  // Trim header/footer
  lines = lines.slice(1, -1);
  // Clear any trailing newlines
  lines = lines.join('\n').trim().split('\n');
// Normalize spacing
  for (i = j = 0, len = lines.length; j < len; i = ++j) {
    line = lines[i];
    lines[i] = '  ' + line.trim();
  }
// Trim satisfied but behind message
  for (i = k = 0, len1 = lines.length; k < len1; i = ++k) {
    line = lines[i];
    if (/The following dependenc/.test(line)) {
      return lines.slice(0, i);
    }
  }
  return lines;
};

// Reads updated deps from output of command
var parseDeps = function(lines) {
  var dep, j, len, results;
  results = [];
  for (j = 0, len = lines.length; j < len; j++) {
    dep = lines[j];
    dep = (dep.trim().split(' ')).shift();
    if (dep === '') {
      continue;
    }
    results.push(dep);
  }
  return results;
};

// Check stdout to see if we need to commit changes
var needsUpdate = function(stdout) {
  return /Upgraded .*package\.json/.test(stdout);
};

// Strip ncu -a message
var stripNcu = function(stdout) {
  var j, len, messages, msg;
  messages = ['\nThe following dependencies are satisfied by their declared version range, but the installed versions are behind. You can install the latest versions without modifying your package file by using npm update. If you want to update the dependencies in your package file anyway, run ncu -a.\n', '\nThe following dependency is satisfied by its declared version range, but the installed version is behind. You can install the latest version without modifying your package file by using npm update. If you want to update the dependency in your package file anyway, run ncu -a.\n'];
  for (j = 0, len = messages.length; j < len; j++) {
    msg = messages[j];
    stdout = stdout.replace(msg, '');
  }
  return stdout;
};

// src/git.coffee
var generateMessage, writeMessage;

// Generate commit message for git
generateMessage = function(stdout) {
  var deps, lines;
  lines = splitLines(stdout);
  deps = parseDeps(lines);
  return `Update ${deps.join(', ')}\n\n${lines.join('\n')}`;
};

// Write commit to temporary file
writeMessage = function(data) {
  return new Promise(function(resolve, reject) {
    return tmp.file(function(err, path$$1, fd) {
      if (err != null) {
        return reject(err);
      }
      return fs.writeFile(fd, data, function(err) {
        if (err != null) {
          return reject(err);
        } else {
          return resolve(path$$1);
        }
      });
    });
  });
};

// Checks whether the local git directory exists
var gitExists = function() {
  return new Promise(function(resolve, reject) {
    return exec.quiet('git rev-parse --git-dir').then(function({stderr}) {
      if (/fatal: Not a git repository/.test(stderr)) {
        return resolve(false);
      } else {
        return resolve(true);
      }
    });
  });
};

// Checks whether the local git working directory is clean or not
var gitOk = function() {
  return new Promise(function(resolve, reject) {
    return gitExists().then(function(exists) {
      if (!exists) {
        return resolve(true);
      }
      return exec.quiet('git status --porcelain').then(function({stderr, stdout}) {
        if (stderr || stdout) {
          console.error(stdout + stderr);
          return reject(new Error('Git working directory not clean'));
        } else {
          return resolve(true);
        }
      });
    });
  });
};

// Commit changes + run npm or yarn update
var gitCommit = function(stdout) {
  return new Promise(function(resolve, reject) {
    // Check if we're in a git repo
    return gitExists().then(function(exists) {
      if (!exists) {
        return resolve(true);
      }
      // Generate message, add files and commit
      return new Promise(function(resolve, reject) {
        var cmds, message, path$$1;
        message = generateMessage(stdout);
        path$$1 = null;
        cmds = [
          'git add .',
          function() {
            return (writeMessage(message)).then(function(v) {
              return path$$1 = v;
            });
          },
          function() {
            return `git commit -F ${path$$1}`;
          }
        ];
        return exec.quiet(cmds).then(function(res) {
          console.log();
          console.log(res.stdout.trim());
          if (res.stderr !== '') {
            console.log(res.stderr.trim());
          }
          return resolve(true);
        }).catch(reject);
      });
    });
  });
};

// src/npm.coffee

// Fix long-standing bug with various versions of npm
var npmFix = function() {
  return new Promise(function(resolve, reject) {
    var err, npmLog, npmPath;
    try {
      require('npm');
      return resolve(true);
    } catch (error) {
      err = error;
      if (/log.gauge.isEnabled/.test(err.stack.toString())) {
        console.log('Attempting to fix npm...');
        npmPath = join(dirname(require.resolve('npm')), '../');
        npmLog = join(npmPath, 'node_modules', 'npmlog');
        return fs.exists(npmLog, function(exists) {
          var cmd;
          if (exists) {
            cmd = `rm -rf ${npmLog}`;
            console.log(cmd);
            return exec(cmd).then(resolve).catch(reject);
          } else {
            return reject('Unable to apply npmfix');
          }
        });
      }
    }
  });
};

// Do yarn upgrade / npm update
var npmInstall = function() {
  if (tasks.has('yarn:upgrade')) {
    return exec('echo\necho $ yarn upgrade\nyarn upgrade');
  } else {
    return exec('echo\necho $ npm install\nnpm install');
  }
};

// src/index.coffee

function index(opts = {}) {
  var ncu, ncuBin, ncuPath;
  if (opts.commit == null) {
    opts.commit = true;
  }
  if (Array.isArray(opts.ignore)) {
    opts.ignore = opts.ignore.join(',');
  } else {
    if (opts.ignore == null) {
      opts.ignore = null;
    }
  }
  // Find path to node-check-updates binary
  // TODO: Figure out why npm does not correctly symlink it's binary
  ncuPath = path.dirname(require.resolve('npm-check-updates-lite'));
  ncuBin = path.join(ncuPath, '../bin/ncu');
  ncu = opts.ignore != null ? `${ncuBin} -x ${opts.ignore}` : ncuBin;
  task('outdated', 'show outdated packages', function*() {
    var status, stderr, stdout;
    if (!(yield npmFix())) {
      return;
    }
    ({stdout, stderr, status} = (yield exec.quiet(ncu)));
    log(stdout, stderr);
    if (status !== 0) {
      return process.exit(status);
    }
  });
  task('outdated:update', 'update outdated packages', function*() {
    var status, stderr, stdout;
    if (!(yield npmFix())) {
      return;
    }
    if (!(yield gitOk())) {
      return;
    }
    ({stdout, stderr, status} = (yield exec.quiet(ncu + ' -u')));
    log(stdout, stderr);
    if (status !== 0) {
      process.exit(status);
    }
    if (needsUpdate(stdout)) {
      yield npmInstall();
      if (opts.commit) {
        return (yield gitCommit(stdout));
      }
    }
  });
  return task('outdated:all', 'update all outdated packages', function*() {
    var status, stderr, stdout;
    if (!(yield npmFix())) {
      return;
    }
    if (!(yield gitOk())) {
      return;
    }
    ({stdout, stderr, status} = (yield exec.quiet(ncu + ' -u -a')));
    // ncu erroneously logs message to use -a even when you use -a, strip that
    stdout = stripNcu(stdout);
    log(stdout, stderr);
    if (status !== 0) {
      process.exit(status);
    }
    if (needsUpdate(stdout)) {
      yield npmInstall();
      if (opts.commit) {
        return (yield gitCommit(stdout));
      }
    }
  });
}

export default index;
//# sourceMappingURL=sake-outdated.es6.js.map
